"use strict";

exports.__esModule = true;
exports.variant = variant;

var _themeValue = require("../values/theme-value");

var _createStyles3 = require("./create-styles");

var _createRule = require("./create-rule");

var VARIANT_PROP = 'variant';
/**
 * Create `variant` from styles defined directly in `theme`.
 * Inspired by [`styled-system`](https://github.com/jxnblk/styled-system).
 *
 * Related: {@link textStyle}, {@link boxStyle}, {@link rule}, {@link themeValue}.
 *
 * @param {Object} options
 * @param {String} [options.prop = 'variant']
 *
 * @example
 * import { variant } from 'pss'
 *
 * const variant = variant({
 *   themeKey: 'textStyle'
 * })
 *
 * const Text = styled.p`
 *   ${variant}
 * `
 *
 * @example
 * const theme = {
 *   textStyle: {
 *     default: {
 *       fontSize: '16px',
 *       lineHeight: 1.2,
 *       fontWeight: normal,
 *       fontFamily: 'system-ui'
 *     },
 *     heading: {
 *       fontSize: '2rem',
 *       lineHeight: 1.2,
 *       fontWeight: 'bold',
 *       fontFamily: 'system-ui'
 *     }
 *   }
 * }
 *
 * <Text variant='default' /> // → `theme.textStyle.default`
 * <Text variant='heading' /> // → `theme.textStyle.heading`
 *
 * @example
 * const themeWithDefault = {
 *   default: {
 *      textStyle: 'default',
 *   },
 *   ...theme,
 * }
 *
 * <Text variant='auto' /> // → `theme.textStyle.default`
 * <Text variant /> // → `theme.textStyle.default`
 */

function variant(_ref) {
  var _createStyles;

  var _ref$prop = _ref.prop,
      prop = _ref$prop === void 0 ? VARIANT_PROP : _ref$prop,
      defaultKeyword = _ref.defaultKeyword,
      themeKey = _ref.themeKey,
      transformValue = _ref.transformValue,
      scale = _ref.scale,
      getter = _ref.getter;
  var rule = (0, _createRule.createRule)({
    getValue: (0, _themeValue.themeValue)({
      defaultKeyword: defaultKeyword,
      themeKey: themeKey,
      transformValue: transformValue,
      scale: scale,
      getter: getter
    })
  });
  var mixin = (0, _createStyles3.createStyles)((_createStyles = {}, _createStyles[prop] = rule, _createStyles));

  if (prop !== VARIANT_PROP) {
    var _createStyles2;

    mixin[VARIANT_PROP] = (0, _createStyles3.createStyles)((_createStyles2 = {}, _createStyles2[VARIANT_PROP] = rule, _createStyles2));
  }

  return mixin;
}