"use strict";

exports.__esModule = true;
exports.themePath = themePath;
exports.getFirst = exports.getFirstKey = exports.floor = exports.splitUnit = exports.wrapIfMedia = exports.wrap = exports.addPrefix = exports.cap = exports.px = exports.rem = exports.isMediaKey = exports.getMedia = exports.getThemeMedia = exports.getDefault = void 0;

var _utils = require("@exah/utils");

var _constants = require("./constants");

var defaultTo = function defaultTo(fallback) {
  return function (input) {
    return input == null ? fallback : input;
  };
};
/**
 * ```js
 * import { themePath } from 'pss'
 * ```
 *
 * Get value from theme. Optionally accept fallback and / or list of transforms.
 *
 * @example
 * import { themePath, px } from 'pss'
 *
 * const Box = styled.div`
 *   width: ${themePath('size.card', px)};
 *   background-color: ${themePath('color.red', 'hotpink')};
 * `
 *
 * <Box /> // → width: 200px; background-color: hotpink;
 */


function themePath(input, defaultValue) {
  var toDefault = (0, _utils.isFn)(defaultValue) ? defaultValue : defaultTo(defaultValue);

  for (var _len = arguments.length, fns = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    fns[_key - 2] = arguments[_key];
  }

  var transform = _utils.pipe.apply(void 0, [toDefault].concat(fns));

  return function (props) {
    return transform((0, _utils.path)(input)(props.theme));
  };
}

var getDefault = function getDefault(input, defaultValue) {
  if (defaultValue === void 0) {
    defaultValue = _constants.DEFAULT_KEY;
  }

  return themePath([_constants.DEFAULT_KEY, input], defaultValue);
};

exports.getDefault = getDefault;
var getThemeMedia = themePath(_constants.THEME_MEDIA_KEY, {});
exports.getThemeMedia = getThemeMedia;

var getMedia = function getMedia(input) {
  return (0, _utils.compose)((0, _utils.path)(input), getThemeMedia);
};

exports.getMedia = getMedia;

var isMediaKey = function isMediaKey(key, media) {
  return key in media || key === _constants.DEFAULT_MEDIA_KEY;
};
/**
 * Convert number to rem
 *
 * @param {number} [base = 16] - root font size
 * @param {number} [input = base] - value for convertion
 *
 * @example
 * import { rem } form 'pss'
 *
 * rem(16, 16) // → 1rem
 * rem(16, 20) // → 1.25rem
 * rem(16, '20px') // → 1.25rem
 *
 * const myRem = rem(20)
 * myRem(16) // → 0.8rem
 */


exports.isMediaKey = isMediaKey;
var rem = (0, _utils.curryN)(2, function (base, input) {
  if (base === void 0) {
    base = 16;
  }

  if (input === void 0) {
    input = base;
  }

  return /rem$/.test(input) ? input : parseFloat(input, 10) / base + "rem";
});
/**
 * Convert numbers to pixels
 *
 * @example
 * import { px } from 'pss'
 *
 * px(30) // → '30px'
 * px(0) // → 0
 * px('100px') // → '100px'
 * px('100rem') // → '100rem' (nothing changed)
 */

exports.rem = rem;

var px = function px(num) {
  return (0, _utils.isNum)(num) && num !== 0 ? num + "px" : num;
};

exports.px = px;

var cap = function cap(input) {
  if (input === void 0) {
    input = '';
  }

  return input.replace(/^\w/, function (c) {
    return c.toUpperCase();
  });
};

exports.cap = cap;

var addPrefix = function addPrefix(prefix, input) {
  return prefix ? prefix + cap(input) : input;
};

exports.addPrefix = addPrefix;
var wrap = (0, _utils.curryN)(2, function (name, value) {
  var _ref;

  return value != null ? name ? (_ref = {}, _ref[name] = value, _ref) : value : null;
});
exports.wrap = wrap;

var wrapIfMedia = function wrapIfMedia(query) {
  for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  return wrap.apply(void 0, [query && "@media " + query].concat(args));
};

exports.wrapIfMedia = wrapIfMedia;

var parseUnit = function parseUnit(str) {
  return str.replace(/([\d.]+)(\D+)?$/, '$2').trim() || undefined;
};
/**
 * @example
 * import { splitUnit } from 'pss'
 *
 * const [ value, unit ] = splitUnit('30px') // → [ 30, 'px' ]
 */


var splitUnit = function splitUnit(input) {
  return (0, _utils.isStr)(input) ? [parseFloat(input, 10), parseUnit(input)] : (0, _utils.isNum)(input) ? [input, undefined] : [];
};

exports.splitUnit = splitUnit;

var floor = function floor(number, precision) {
  if (precision === void 0) {
    precision = 0;
  }

  var factor = Math.pow(10, precision);
  return Math.floor(number * factor) / factor;
};

exports.floor = floor;

var getFirstKey = function getFirstKey(input) {
  return Object.keys(Object(input))[0];
};

exports.getFirstKey = getFirstKey;

var getFirst = function getFirst(input) {
  if ((0, _utils.isObj)(input)) {
    if (Object.prototype.hasOwnProperty.call(input, _constants.DEFAULT_KEY)) {
      return input["default"];
    }

    var firstKey = getFirstKey(input);
    return firstKey && input[firstKey];
  }

  return input;
};

exports.getFirst = getFirst;