"use strict";

exports.__esModule = true;
exports.createSpaceValue = createSpaceValue;
exports.spaceValue = void 0;

var _utils = require("@exah/utils");

var _constants = require("../constants");

var _utils2 = require("../utils");

var getSpaceStep = function getSpaceStep(input, spaces) {
  if (spaces === void 0) {
    spaces = [];
  }

  var value = spaces[Math.abs(input)];
  var coeficent = input < 0 ? -1 : 1;

  if (value === undefined) {
    return null;
  } else if ((0, _utils.isStr)(value)) {
    var _splitUnit = (0, _utils2.splitUnit)(value),
        number = _splitUnit[0],
        _splitUnit$ = _splitUnit[1],
        unit = _splitUnit$ === void 0 ? 'px' : _splitUnit$;

    return number * coeficent + unit;
  }

  return value * coeficent;
};

function createSpaceValue(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$themeKey = _ref.themeKey,
      themeKey = _ref$themeKey === void 0 ? 'space' : _ref$themeKey,
      _ref$scale = _ref.scale,
      scale = _ref$scale === void 0 ? [0, 4, 8, 16, 32, 64, 128, 256] : _ref$scale,
      _ref$transformValue = _ref.transformValue,
      transformValue = _ref$transformValue === void 0 ? _utils.identity : _ref$transformValue,
      _ref$getter = _ref.getter,
      getter = _ref$getter === void 0 ? (0, _utils2.themePath)(themeKey, scale) : _ref$getter;

  var transform = (0, _utils.pipe)(getSpaceStep, transformValue);
  return function (defaultValue) {
    return function (input, props, mediaKey) {
      if ((0, _utils.isNum)(input) || (0, _utils.isBool)(input)) {
        var spaces = getter(props);

        if ((0, _utils.isArr)(spaces)) {
          return transform(input, spaces);
        }

        if (mediaKey != null) {
          return transform(input, (0, _utils.fallbackTo)(spaces[mediaKey], spaces[_constants.DEFAULT_MEDIA_KEY]));
        }

        return (0, _utils.mapObj)(function (key, value) {
          var _ref2;

          return _ref2 = {}, _ref2[key] = transform(input, value), _ref2;
        }, spaces);
      }

      return defaultValue;
    };
  };
}
/**
 * ```js
 * import { spaceValue } from 'pss'
 * ```
 *
 * Spacing system for `margin`, `padding`. Default behaviour described in {@link space}.
 * Must be used with {@link rule}.
 *
 * Related: {@link space}, {@link sizes}, {@link rule}, {@link sizeValue}.
 *
 * @param [defaultValue = sizeValue(identity)] — Fallback value used when prop value is {@link String} or nothing returned.
 * @return {Function} - that must be used in {@link rule}
 *
 * @example
 * import pss, { rule, spaceValue } from 'pss'
 *
 * const spaceRule = (name) => rule(name, spaceValue())
 *
 * const margin = pss({
 *   m: spaceRule('margin'),
 *   ml: spaceRule('marginLeft'),
 *   mr: spaceRule('marginRight'),
 *   mt: spaceRule('marginTop'),
 *   mb: spaceRule('marginBottom'),
 *   mx: [ spaceRule('marginLeft'), spaceRule('marginRight') ],
 *   my: [ spaceRule('marginTop'), spaceRule('marginBottom') ]
 * })
 *
 * const Box = styled.div`
 *   ${margin}
 * `
 *
 * @example
 * const theme = {
 *   media: {
 *     sm: '(max-width: 600px)' // optional
 *   },
 *   space: [ 0, 8, 16, 32, 64 ]
 * }
 *
 * <ThemeProvider theme={theme}>
 *   <Box m={1} /> // → margin: 8px;
 *   <Box mx={2} /> // → margin-left: 16px; margin-right: 16px
 *   <Box m={{ sm: 1 }} /> // → @media (max-width: 600px) { margin: 8px }
 * </ThemeProvider>
 *
 * @example
 * const theme = {
 *   media: {
 *     sm: '(max-width: 600px)'
 *   },
 *   space: {
 *     all: [ 0, 8, 16, 32, 64 ],
 *     sm: [ 0, 4, 8, 16, 32 ]
 *   }
 * }
 *
 * <ThemeProvider theme={theme}>
 *   <Box m={1} /> // → margin: 8px; @media (max-width: 600px) { margin: 4px }
 *   <Box mx={2} /> // → margin-left: 16px; margin-right: 16px; @media (max-width: 600px) { margin-left: 8px; margin-right: 8px; }
 *   <Box m={{ sm: 1 }} /> // → @media (max-width: 600px) { margin: 4px }
 * </ThemeProvider>
 */


var spaceValue = createSpaceValue({
  transformValue: _utils2.px
});
exports.spaceValue = spaceValue;