import { identity, fallbackTo, isStr, path } from '@exah/utils';
import { themePath, getDefault, getFirst } from '../utils';
export function createColorValue(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$themeColorKey = _ref.themeColorKey,
      themeColorKey = _ref$themeColorKey === void 0 ? 'color' : _ref$themeColorKey,
      _ref$themePaletteKey = _ref.themePaletteKey,
      themePaletteKey = _ref$themePaletteKey === void 0 ? 'palette' : _ref$themePaletteKey,
      _ref$colorsScale = _ref.colorsScale,
      colorsScale = _ref$colorsScale === void 0 ? {} : _ref$colorsScale,
      _ref$paletteScale = _ref.paletteScale,
      paletteScale = _ref$paletteScale === void 0 ? {} : _ref$paletteScale,
      _ref$colorsGetter = _ref.colorsGetter,
      colorsGetter = _ref$colorsGetter === void 0 ? themePath(themeColorKey, colorsScale) : _ref$colorsGetter,
      _ref$paletteGetter = _ref.paletteGetter,
      paletteGetter = _ref$paletteGetter === void 0 ? themePath(themePaletteKey, paletteScale) : _ref$paletteGetter,
      _ref$defaultKeyword = _ref.defaultKeyword,
      defaultKeyword = _ref$defaultKeyword === void 0 ? 'auto' : _ref$defaultKeyword;

  var getColor = function getColor(defaultColorName) {
    return function (colorName, props) {
      var colors = colorsGetter(props);
      var palettes = paletteGetter(props);
      var defaultPaletteName = getDefault(themePaletteKey)(props);
      var activeColors = Object.assign({}, path(defaultPaletteName)(palettes), {}, colors);
      var color = colorName === true || colorName === defaultKeyword ? path(defaultColorName)(activeColors) : isStr(colorName) ? path(colorName)(activeColors) : null;

      if (colorName == null) {
        return color;
      }

      return fallbackTo(color, path(defaultColorName)(path(colorName)(palettes)));
    };
  };

  return function (key, transformValue, defaultValue) {
    if (transformValue === void 0) {
      transformValue = identity;
    }

    var getValue = getColor(key);
    return function (input, props) {
      var color = getFirst(getValue(input, props));
      return fallbackTo(isStr(color) ? transformValue(color, props) : color, defaultValue);
    };
  };
}
/**
 * ```js
 * import { colorValue } from 'pss'
 * ```
 *
 * Get color from theme and apply it to css prop. Must be used with {@link rule}.
 *
 * @param {string} key — Key in `theme.color` or in `theme.palette[theme.default.palette]`
 * @param {Function} [transformValue = identity] — Return customized CSS prop value (i.e. `box-shadow`, gradients)
 *
 * @return {Function}
 *
 * @example
 * import pss, { rule, colorValue } from 'pss'
 *
 * const colors = pss({
 *   fg: rule('color', colorValue('fg')),
 *   bg: rule('backgroundColor', colorValue('bg')),
 *   shadow: rule('boxShadow', colorValue('shadow', (color) => `0 0 20px 0 ${color}`)),
 *   tm: [
 *      rule('color', colorValue('fg')),
 *      rule('backgroundColor', colorValue('bg'))
 *   ]
 * })
 *
 * // Add to component
 * const Box = styled.div`
 *   ${colors}
 * `
 *
 * @example
 * // theme.palette.default.fg
 * <Box fg='auto' /> // background-color: #222222
 * <Box fg /> // background-color: #222222
 *
 * // theme.colors.black
 * <Box fg='black' /> // color: #222222
 *
 * // theme.palette.default.accent
 * <Box fg='accent' /> // color: #ff0000
 *
 * // theme.palette.default.shadow
 * <Box shadow='auto' /> // box-shadow: 0 0 20px 0 rgba(0, 0, 0, 0.2)
 * <Box shadow /> // box-shadow: 0 0 20px 0 rgba(0, 0, 0, 0.2)
 *
 * // theme.palette.default.fg, theme.palette.default.bg
 * <Box tm='default' /> // color: #222222; background-color: #ffffff
 */

export var colorValue = createColorValue();