import { path, identity, isObj, mapObj } from '@exah/utils';
import { getDefault, themePath } from '../utils';
/**
 * ```js
 * import { themeValue } from 'pss'
 * ```
 *
 * Use values defined in `theme[themeKey]`.
 *
 * See {@link fontFamily}, {@link radius}.
 *
 * @param {Object} [options = {}]
 * @return {Function}
 *
 * @example
 * const theme = {
 *   font: {
 *     heading: 'Times New Roman, serif',
 *     ui: 'system-ui, Helvetica, sans-serif'
 *   }
 * }
 *
 * @example
 * import pss, { themeValue } from 'pss'
 *
 * const Text = styled.div(pss({
 *   fontFamily: rule('fontFamily', themeValue({ themeKey: 'font' }))
 * }))
 *
 * <ThemeProvider theme={theme}>
 *   <Text fontFamily='ui'> // → font-family: system-ui, Helvetica, sans-serif
 *     Hello World!
 *   </Text>
 * </ThemeProvider>
 */

export function themeValue(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      themeKey = _ref.themeKey,
      _ref$transformValue = _ref.transformValue,
      transformValue = _ref$transformValue === void 0 ? identity : _ref$transformValue,
      fallback = _ref.fallback,
      _ref$scale = _ref.scale,
      scale = _ref$scale === void 0 ? {} : _ref$scale,
      _ref$defaultKeyword = _ref.defaultKeyword,
      defaultKeyword = _ref$defaultKeyword === void 0 ? 'auto' : _ref$defaultKeyword;

  var getThemeScale = themePath(themeKey, scale);
  return function (defaultValue) {
    if (defaultValue === void 0) {
      defaultValue = fallback;
    }

    return function (input, props, mediaKey) {
      var isDefaultValue = input === true || defaultKeyword != null && input === defaultKeyword;
      var valueKey = isDefaultValue ? getDefault(themeKey)(props) : input;
      var themeScale = getThemeScale(props);
      var result = path(valueKey, defaultValue)(themeScale);

      if (isObj(result)) {
        if (Object.prototype.hasOwnProperty.call(result, mediaKey)) {
          return transformValue(result[mediaKey]);
        }

        return mapObj(function (key, val) {
          var _ref2;

          return _ref2 = {}, _ref2[key] = transformValue(val), _ref2;
        }, result);
      }

      return transformValue(result);
    };
  };
}